@@ -11,6 +11,7 @@
 #include <QtCore/QRegularExpression>
 
 #include <iostream>
+#include <optional>
 
 QT_BEGIN_NAMESPACE
 
@@ -120,6 +121,7 @@ private:
         Tok_Equals,
         Tok_LeftBracket,
         Tok_RightBracket,
+        Tok_Attribute,
         Tok_LeftAngleBracket,
         Tok_RightAngleBracket,
         Tok_QuestionMark,
@@ -178,9 +180,10 @@ private:
     bool qualifyOneCallbackOwn(const Namespace *ns, void *context) const;
     bool qualifyOneCallbackUsing(const Namespace *ns, void *context) const;
     bool qualifyOne(const NamespaceList &namespaces, int nsCnt, const HashString &segment,
-                    NamespaceList *resolved, QSet<HashStringList> *visitedUsings) const;
+                    NamespaceList *resolved, Namespace const **resolvedNamespace,
+                    QSet<HashStringList> *visitedUsings, bool *foundViaUsing = nullptr) const;
     bool qualifyOne(const NamespaceList &namespaces, int nsCnt, const HashString &segment,
-                    NamespaceList *resolved) const;
+                    NamespaceList *resolved, Namespace const **resolvedNamespace) const;
     bool fullyQualify(const NamespaceList &namespaces, int nsCnt,
                       const NamespaceList &segments, bool isDeclaration,
                       NamespaceList *resolved, NamespaceList *unresolved) const;
@@ -884,10 +887,45 @@ CppParser::TokenType CppParser::getToken()
                 yyCh = getChar();
                 return Tok_RightParen;
             case '[':
+                yyCh = getChar();
+                if (yyCh == '[') {
+                    // C++11 attribute: [[...]]
+                    yyCh = getChar();
+                    int depth = 1;
+                    bool inString = false;
+                    bool inChar = false;
+                    bool escaped = false;
+
+                    // Need to handle nested brackets, strings, and character literals
+                    while (depth > 0 && yyCh != EOF) {
+                        if (escaped) {
+                            escaped = false;
+                        } else if (yyCh == '\\') {
+                            escaped = true;
+                        } else if (yyCh == '"' && !inChar) {
+                            inString = !inString;
+                        } else if (yyCh == '\'' && !inString) {
+                            inChar = !inChar;
+                        } else if (!inString && !inChar) {
+                            if (yyCh == '[')
+                                depth++;
+                            else if (yyCh == ']')
+                                depth--;
+                        }
+                        yyCh = getChar();
+                    }
+
+                    if (yyCh == EOF || yyCh != ']') {
+                        yyMsg(yyCurLineNo) << "Unterminated C++ attribute (missing ']]')\n";
+                        return Tok_LeftBracket;
+                    }
+
+                    yyCh = getChar();
+                    return Tok_Attribute;
+                }
                 if (yyBracketDepth == 0)
                     yyBracketLineNo = yyCurLineNo;
                 yyBracketDepth++;
-                yyCh = getChar();
                 return Tok_LeftBracket;
             case ']':
                 if (yyBracketDepth == 0)
@@ -1024,8 +1062,13 @@ bool CppParser::visitNamespace(const NamespaceList &namespaces, int nsCount,
 
 struct QualifyOneData {
     QualifyOneData(const NamespaceList &ns, int nsc, const HashString &seg, NamespaceList *rslvd,
-                   QSet<HashStringList> *visited)
-        : namespaces(ns), nsCount(nsc), segment(seg), resolved(rslvd), visitedUsings(visited)
+                   QSet<HashStringList> *visited, Namespace const **resolvedNs)
+        : namespaces(ns),
+          nsCount(nsc),
+          segment(seg),
+          resolved(rslvd),
+          visitedUsings(visited),
+          resolvedNamespace(resolvedNs)
     {}
 
     const NamespaceList &namespaces;
@@ -1033,14 +1076,16 @@ struct QualifyOneData {
     const HashString &segment;
     NamespaceList *resolved;
     QSet<HashStringList> *visitedUsings;
+    Namespace const **resolvedNamespace;
 };
 
 bool CppParser::qualifyOneCallbackOwn(const Namespace *ns, void *context) const
 {
     QualifyOneData *data = (QualifyOneData *)context;
-    if (ns->children.contains(data->segment)) {
+    if (auto rns = ns->children.constFind(data->segment); rns != ns->children.cend()) {
         *data->resolved = data->namespaces.mid(0, data->nsCount);
         *data->resolved << data->segment;
+        *data->resolvedNamespace = *rns;
         return true;
     }
     auto nsai = ns->aliases.constFind(data->segment);
@@ -1057,6 +1102,7 @@ bool CppParser::qualifyOneCallbackOwn(const Namespace *ns, void *context) const
             nslIn = nslOut;
         }
         *data->resolved = nsl;
+        *data->resolvedNamespace = ns;
         return true;
     }
     return false;
@@ -1069,29 +1115,39 @@ bool CppParser::qualifyOneCallbackUsing(const Namespace *ns, void *context) cons
         if (!data->visitedUsings->contains(use)) {
             data->visitedUsings->insert(use);
             if (qualifyOne(use.value(), use.value().size(), data->segment, data->resolved,
-                           data->visitedUsings))
+                           data->resolvedNamespace, data->visitedUsings))
                 return true;
         }
     return false;
 }
 
 bool CppParser::qualifyOne(const NamespaceList &namespaces, int nsCnt, const HashString &segment,
-                           NamespaceList *resolved, QSet<HashStringList> *visitedUsings) const
+                           NamespaceList *resolved, Namespace const **resolvedNamespace,
+                           QSet<HashStringList> *visitedUsings, bool *foundViaUsing) const
 {
-    QualifyOneData data(namespaces, nsCnt, segment, resolved, visitedUsings);
+    QualifyOneData data(namespaces, nsCnt, segment, resolved, visitedUsings, resolvedNamespace);
 
-    if (visitNamespace(namespaces, nsCnt, &CppParser::qualifyOneCallbackOwn, &data))
+    if (visitNamespace(namespaces, nsCnt, &CppParser::qualifyOneCallbackOwn, &data)) {
+        if (foundViaUsing)
+            *foundViaUsing = false;
         return true;
+    }
 
-    return visitNamespace(namespaces, nsCnt, &CppParser::qualifyOneCallbackUsing, &data);
+    if (visitNamespace(namespaces, nsCnt, &CppParser::qualifyOneCallbackUsing, &data)) {
+        if (foundViaUsing)
+            *foundViaUsing = true;
+        return true;
+    }
+
+    return false;
 }
 
 bool CppParser::qualifyOne(const NamespaceList &namespaces, int nsCnt, const HashString &segment,
-                           NamespaceList *resolved) const
+                           NamespaceList *resolved, Namespace const **resolvedNamespace) const
 {
     QSet<HashStringList> visitedUsings;
 
-    return qualifyOne(namespaces, nsCnt, segment, resolved, &visitedUsings);
+    return qualifyOne(namespaces, nsCnt, segment, resolved, resolvedNamespace, &visitedUsings);
 }
 
 bool CppParser::fullyQualify(const NamespaceList &namespaces, int nsCnt,
@@ -1115,38 +1171,97 @@ bool CppParser::fullyQualify(const NamespaceList &namespaces, int nsCnt,
         nsIdx = nsCnt - 1;
     }
 
+    // Check if the identifier to resolve shares a common suffix
+    // with the current namespace path. This handles cases like:
+    //   - Current scope: ["", "Outer", "Test"]
+    //   - Trying to resolve: ["Outer", "Test"]
+
     auto matchSeg = segments.crbegin();
     auto matchNs = namespaces.crbegin();
 
     if (matchSeg->value() != matchNs->value())
         matchSeg++;
 
+    int matchingSuffixLength = 0;
     while (matchSeg != segments.crend()
            && matchNs != namespaces.crend()
            && matchSeg->value() == matchNs->value()) {
 
         matchSeg++;
         matchNs++;
-        nsIdx--;
+        matchingSuffixLength++;
     }
 
+    // The main loop to resolve the segments in the current namespace.
+    // There are three possible outcomes:
+    //      1. resolution of the segments into a class that has tr
+    //         functions (hasTrFunction) if available (trCandidate).
+    //      2. resolution of the segments into a namespace/class without a
+    //         tr function -> in this case, if we're in a handleTr (or
+    //         similar) function, the caller issues a warning (noTrCandidate).
+    //      3. the segments cannot be resolved in the current namespace -> in
+    //         this case, we calculate the non-resolved part (unresolvedCandidate)
+
+    bool viaUsing = false;
+    std::optional<NamespaceList> trCandidate;
+    std::optional<NamespaceList> noTrCandidate;
+    std::optional<NamespaceList> unresolvedCandidate;
     do {
-        if (qualifyOne(namespaces, nsIdx + 1, segments[initSegIdx], resolved)) {
-            int segIdx = initSegIdx;
-            while (++segIdx < segments.size()) {
-                if (!qualifyOne(*resolved, resolved->size(), segments[segIdx], resolved)) {
-                    if (unresolved)
-                        *unresolved = segments.mid(segIdx);
-                    return false;
-                }
+        Namespace const *resolvedNs = nullptr;
+        int segIdx = initSegIdx;
+        int resolveSize = nsIdx + 1;
+        *resolved = namespaces;
+        do {
+            QSet<HashStringList> visitedUsings;
+            if (!qualifyOne(*resolved, resolveSize, segments[segIdx], resolved, &resolvedNs,
+                            &visitedUsings, &viaUsing))
+                break; // could not resolve
+
+            segIdx++;
+            resolveSize = resolved->size();
+        } while (segIdx < segments.size());
+
+        if (segIdx == segments.size()) { // resolved
+            if (resolvedNs && resolvedNs->classDef && resolvedNs->classDef->hasTrFunctions) {
+                if (!viaUsing)
+                    return true;
+                else if (!trCandidate)
+                    trCandidate.emplace(*resolved);
+            } else if (resolvedNs) {
+                if (!viaUsing)
+                    noTrCandidate.emplace(*resolved);
+                else if (!noTrCandidate)
+                    noTrCandidate.emplace(*resolved);
             }
-            return true;
+        } else if (!unresolvedCandidate) { // unresolved
+            unresolvedCandidate.emplace(segments.mid(segIdx));
         }
-    } while (!isDeclaration && --nsIdx >= 0);
+
+        // For declarations: limit search to suffix-matched depths
+        // For usages: search all parent levels (standard C++ lookup)
+    } while ((!isDeclaration || matchingSuffixLength-- > 0) && --nsIdx >= 0);
+
+    // Depending on the case, after the loop, we might have only one of the
+    // outcome candidates, or 2 or 3 (all possbile outcome candidates).
+    // We prioritize the candidates as follows:
+    // 1. if a class with tr commands is found (trCandidate), we take it
+    //    as the result.
+    // 2. otherwise, if any class/namespace without tr commands is
+    //    found (noTrCandidate), take it as the result.
+    // 3. Finally, if none of the above candidates are found, we return the
+    //    unresolved part (unresolvedCandidate).
+
+    if (trCandidate) { // resolved class with tr functions gets priority
+        *resolved = *trCandidate;
+        return true;
+    } else if (noTrCandidate) { // the non-tr resolved namespace/class
+        *resolved = *noTrCandidate;
+        return true;
+    } else if (unresolvedCandidate && unresolved) {
+        *unresolved = *unresolvedCandidate;
+    }
+
     resolved->clear();
-    *resolved << HashString(QString());
-    if (unresolved)
-        *unresolved = segments.mid(initSegIdx);
     return false;
 }
 
@@ -1191,7 +1306,7 @@ void CppParser::enterNamespace(NamespaceList *namespaces, const HashString &name
         ns = modifyNamespace(namespaces, false);
 
     const Namespace *cns = &results->rootNamespace;
-    for (int i = 0; i < namespaces->size(); ++i) {
+    for (int i = 1; i < namespaces->size(); ++i) {
         ns->usings << cns->usings;
         if (!(cns = cns->children.value(namespaces->at(i))))
             break;
@@ -1878,6 +1993,8 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                         // which case 'QMessageBox' is the class name, not 'Q_EXPORT', by
                         // abandoning any qualification collected so far.
                         quali.clear();
+                    } else if (yyTok == Tok_Attribute) {
+                        yyTok = getToken();
                     } else {
                         break;
                     }
@@ -1938,6 +2055,9 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             break;
         case Tok_namespace:
             yyTok = getToken();
+            // Skip C++11 attributes on namespace
+            while (yyTok == Tok_Attribute)
+                yyTok = getToken();
             if (yyTok == Tok_Ident) {
                 QString text = yyWord;
                 text.detach();
@@ -2059,6 +2179,13 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                 prefix += yyWord;
                 prefix.detach();
             } else {
+                // Breaking the :: chain
+                if (!prefix.isEmpty() && yyTok != Tok_LeftParen && yyTok != Tok_LeftBracket
+                    && yyTok != Tok_Equals) {
+                    // We're breaking the chain NOT at a function call or member initialization
+                    // This means prospectiveContext was from a return/member type
+                    prospectiveContext.clear();
+                }
                 prefix.clear();
             }
             metaExpected = false;
