@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qdbusviewer.h"
 #include "qdbusmodel.h"
@@ -45,6 +20,7 @@
 #include <QtDBus/QDBusConnectionInterface>
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusMetaType>
+#include <QtDBus/QDBusServiceWatcher>
 
 #include <QtGui/QAction>
 #include <QtGui/QKeyEvent>
@@ -56,6 +32,8 @@
 
 #include <private/qdbusutil_p.h>
 
+using namespace Qt::StringLiterals;
+
 class QDBusViewModel: public QDBusModel
 {
 public:
@@ -87,10 +65,8 @@ public:
     }
 };
 
-QDBusViewer::QDBusViewer(const QDBusConnection &connection, QWidget *parent)  :
-    QWidget(parent),
-    c(connection),
-    objectPathRegExp(QLatin1String("\\[ObjectPath: (.*)\\]"))
+QDBusViewer::QDBusViewer(const QDBusConnection &connection, QWidget *parent)
+    : QWidget(parent), c(connection), objectPathRegExp("\\[ObjectPath: (.*)\\]"_L1)
 {
     serviceFilterLine = new QLineEdit(this);
     serviceFilterLine->setPlaceholderText(tr("Search..."));
@@ -141,6 +117,7 @@ QDBusViewer::QDBusViewer(const QDBusConnection &connection, QWidget *parent)  :
 
     QWidget *servicesWidget = new QWidget;
     QVBoxLayout *servicesLayout = new QVBoxLayout(servicesWidget);
+    servicesLayout->setContentsMargins(QMargins());
     servicesLayout->addWidget(serviceFilterLine);
     servicesLayout->addWidget(servicesView);
     splitter->addWidget(servicesWidget);
@@ -152,23 +129,30 @@ QDBusViewer::QDBusViewer(const QDBusConnection &connection, QWidget *parent)  :
     connect(servicesView->selectionModel(), &QItemSelectionModel::currentChanged, this, &QDBusViewer::serviceChanged);
     connect(tree, &QWidget::customContextMenuRequested, this, &QDBusViewer::showContextMenu);
 
-    QMetaObject::invokeMethod(this, "refresh", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, &QDBusViewer::refresh, Qt::QueuedConnection);
 
     if (c.isConnected()) {
-        logMessage(QLatin1String("Connected to D-Bus."));
-        QDBusConnectionInterface *iface = c.interface();
-        connect(iface, &QDBusConnectionInterface::serviceRegistered, this, &QDBusViewer::serviceRegistered);
-        connect(iface, &QDBusConnectionInterface::serviceUnregistered, this, &QDBusViewer::serviceUnregistered);
-        connect(iface, &QDBusConnectionInterface::serviceOwnerChanged, this, &QDBusViewer::serviceOwnerChanged);
+        QDBusServiceWatcher *watcher =
+                new QDBusServiceWatcher("*", c, QDBusServiceWatcher::WatchForOwnerChange, this);
+        connect(watcher, &QDBusServiceWatcher::serviceOwnerChanged, this,
+                &QDBusViewer::serviceOwnerChanged);
+        logMessage(tr("Connected to D-Bus."));
     } else {
-        logError(QLatin1String("Cannot connect to D-Bus: ") + c.lastError().message());
+        logError(tr("Cannot connect to D-Bus: %1").arg(c.lastError().message()));
     }
 
     objectPathRegExp.setPatternOptions(QRegularExpression::InvertedGreedinessOption);
 }
 
-static inline QString topSplitterStateKey() { return QStringLiteral("topSplitterState"); }
-static inline QString splitterStateKey() { return QStringLiteral("splitterState"); }
+static inline QString topSplitterStateKey()
+{
+    return u"topSplitterState"_s;
+}
+
+static inline QString splitterStateKey()
+{
+    return u"splitterState"_s;
+}
 
 void QDBusViewer::saveState(QSettings *settings) const
 {
@@ -184,7 +168,7 @@ void QDBusViewer::restoreState(const QSettings *settings)
 
 void QDBusViewer::logMessage(const QString &msg)
 {
-    log->append(msg + QLatin1Char('\n'));
+    log->append(msg + '\n'_L1);
 }
 
 void QDBusViewer::showEvent(QShowEvent *)
@@ -209,7 +193,7 @@ bool QDBusViewer::eventFilter(QObject *obj, QEvent *event)
 
 void QDBusViewer::logError(const QString &msg)
 {
-    log->append(QLatin1String("<font color=\"red\">Error: </font>") + msg.toHtmlEscaped() + QLatin1String("<br>"));
+    log->append(tr("<font color=\"red\">Error: </font>%1<br>").arg(msg.toHtmlEscaped()));
 }
 
 void QDBusViewer::refresh()
@@ -253,7 +237,8 @@ void QDBusViewer::activate(const QModelIndex &item)
 
 void QDBusViewer::getProperty(const BusSignature &sig)
 {
-    QDBusMessage message = QDBusMessage::createMethodCall(sig.mService, sig.mPath, QLatin1String("org.freedesktop.DBus.Properties"), QLatin1String("Get"));
+    QDBusMessage message = QDBusMessage::createMethodCall(
+            sig.mService, sig.mPath, "org.freedesktop.DBus.Properties"_L1, "Get"_L1);
     QList<QVariant> arguments;
     arguments << sig.mInterface << sig.mName;
     message.setArguments(arguments);
@@ -280,7 +265,8 @@ void QDBusViewer::setProperty(const BusSignature &sig)
         return;
     }
 
-    QDBusMessage message = QDBusMessage::createMethodCall(sig.mService, sig.mPath, QLatin1String("org.freedesktop.DBus.Properties"), QLatin1String("Set"));
+    QDBusMessage message = QDBusMessage::createMethodCall(
+            sig.mService, sig.mPath, "org.freedesktop.DBus.Properties"_L1, "Set"_L1);
     QList<QVariant> arguments;
     arguments << sig.mInterface << sig.mName << QVariant::fromValue(QDBusVariant(value));
     message.setArguments(arguments);
@@ -305,7 +291,7 @@ void QDBusViewer::callMethod(const BusSignature &sig)
     QMetaMethod method;
     for (int i = 0; i < mo->methodCount(); ++i) {
         const QString signature = QString::fromLatin1(mo->method(i).methodSignature());
-        if (signature.startsWith(sig.mName) && signature.at(sig.mName.length()) == QLatin1Char('('))
+        if (signature.startsWith(sig.mName) && signature.at(sig.mName.size()) == '('_L1)
             if (getDbusSignature(mo->method(i)) == sig.mTypeSig)
                 method = mo->method(i);
     }
@@ -322,7 +308,7 @@ void QDBusViewer::callMethod(const BusSignature &sig)
     const QList<QByteArray> paramTypes = method.parameterTypes();
     const QList<QByteArray> paramNames = method.parameterNames();
     QList<int> types; // remember the low-level D-Bus type
-    for (int i = 0; i < paramTypes.count(); ++i) {
+    for (int i = 0; i < paramTypes.size(); ++i) {
         const QByteArray paramType = paramTypes.at(i);
         if (paramType.endsWith('&'))
             continue; // ignore OUT parameters
@@ -343,7 +329,7 @@ void QDBusViewer::callMethod(const BusSignature &sig)
 
     // Try to convert the values we got as closely as possible to the
     // dbus signature. This is especially important for those input as strings
-    for (int i = 0; i < args.count(); ++i) {
+    for (int i = 0; i < args.size(); ++i) {
         QVariant a = args.at(i);
         int desttype = types.at(i);
         if (desttype < int(QMetaType::User) && desttype != qMetaTypeId<QVariantMap>()) {
@@ -393,9 +379,13 @@ void QDBusViewer::showContextMenu(const QPoint &point)
         menu.addAction(action);
         break; }
     case QDBusModel::PropertyItem: {
+        QDBusInterface iface(sig.mService, sig.mPath, sig.mInterface, c);
+        QMetaProperty prop = iface.metaObject()->property(iface.metaObject()->indexOfProperty(sig.mName.toLatin1()));
         QAction *actionSet = new QAction(tr("&Set value"), &menu);
         actionSet->setData(3);
+        actionSet->setEnabled(prop.isWritable());
         QAction *actionGet = new QAction(tr("&Get value"), &menu);
+        actionGet->setEnabled(prop.isReadable());
         actionGet->setData(4);
         menu.addAction(actionSet);
         menu.addAction(actionGet);
@@ -439,46 +429,45 @@ void QDBusViewer::connectionRequested(const BusSignature &sig)
 void QDBusViewer::dumpMessage(const QDBusMessage &message)
 {
     QList<QVariant> args = message.arguments();
-    QString out = QLatin1String("Received ");
 
+    QString messageType;
     switch (message.type()) {
     case QDBusMessage::SignalMessage:
-        out += QLatin1String("signal ");
+        messageType = tr("signal");
         break;
     case QDBusMessage::ErrorMessage:
-        out += QLatin1String("error message ");
+        messageType = tr("error message");
         break;
     case QDBusMessage::ReplyMessage:
-        out += QLatin1String("reply ");
+        messageType = tr("reply");
         break;
     default:
-        out += QLatin1String("message ");
+        messageType = tr("message");
         break;
     }
 
-    out += QLatin1String("from ");
-    out += message.service();
+    QString out = tr("Received %1 from %2").arg(messageType).arg(message.service());
+
     if (!message.path().isEmpty())
-        out += QLatin1String(", path ") + message.path();
+        out += tr(", path %1").arg(message.path());
     if (!message.interface().isEmpty())
-        out += QLatin1String(", interface <i>") + message.interface() + QLatin1String("</i>");
+        out += tr(", interface <i>%1</i>").arg(message.interface());
     if (!message.member().isEmpty())
-        out += QLatin1String(", member ") + message.member();
-    out += QLatin1String("<br>");
+        out += tr(", member %1").arg(message.member());
+    out += "<br>"_L1;
     if (args.isEmpty()) {
-        out += QLatin1String("&nbsp;&nbsp;(no arguments)");
+        out += tr("&nbsp;&nbsp;(no arguments)");
     } else {
-        out += QLatin1String("&nbsp;&nbsp;Arguments: ");
-        for (const QVariant &arg : qAsConst(args)) {
+        QStringList argStrings;
+        for (const QVariant &arg : std::as_const(args)) {
             QString str = QDBusUtil::argumentToString(arg).toHtmlEscaped();
             // turn object paths into clickable links
-            str.replace(objectPathRegExp, QLatin1String("[ObjectPath: <a href=\"qdbus://bus\\1\">\\1</a>]"));
+            str.replace(objectPathRegExp, tr("[ObjectPath: <a href=\"qdbus://bus\\1\">\\1</a>]"));
             // convert new lines from command to proper HTML line breaks
-            str.replace(QStringLiteral("\n"), QStringLiteral("<br/>"));
-            out += str;
-            out += QLatin1String(", ");
+            str.replace("\n"_L1, "<br/>"_L1);
+            argStrings.append(str);
         }
-        out.chop(2);
+        out += tr("&nbsp;&nbsp;Arguments: %1").arg(argStrings.join(tr(", ")));
     }
 
     log->append(out);
@@ -521,14 +510,6 @@ static QModelIndex findItem(QStringListModel *servicesModel, const QString &name
     return hits.first();
 }
 
-void QDBusViewer::serviceUnregistered(const QString &name)
-{
-    QModelIndex hit = findItem(servicesModel, name);
-    if (!hit.isValid())
-        return;
-    servicesModel->removeRows(hit.row(), 1);
-}
-
 void QDBusViewer::serviceOwnerChanged(const QString &name, const QString &oldOwner,
                                       const QString &newOwner)
 {
@@ -563,7 +544,7 @@ void QDBusViewer::refreshChildren()
 
 void QDBusViewer::anchorClicked(const QUrl &url)
 {
-    if (url.scheme() != QLatin1String("qdbus"))
+    if (url.scheme() != "qdbus"_L1)
         // not ours
         return;
 
