@@ -11,13 +11,12 @@
 #include "config.h"
 #include "enumnode.h"
 #include "functionnode.h"
+#include "genustypes.h"
 #include "htmlgenerator.h"
 #include "node.h"
 #include "qdocdatabase.h"
 #include "typedefnode.h"
 
-#include <QtCore/qcryptographichash.h>
-#include <QtCore/qdebug.h>
 #include <QtCore/qhash.h>
 
 QT_BEGIN_NAMESPACE
@@ -88,6 +87,8 @@ void HelpProjectWriter::reset(const QString &defaultFileName, Generator *g)
             subproject.m_sortPages = config.get(subprefix + "sortPages").asBool();
             subproject.m_type = config.get(subprefix + "type").asString();
             readSelectors(subproject, config.get(subprefix + "selectors").asStringList());
+            subprefix.chop(1);
+            subproject.m_prefix = std::move(subprefix); // Stored for error reporting purposes
             project.m_subprojects.append(subproject);
         }
 
@@ -103,50 +104,51 @@ void HelpProjectWriter::reset(const QString &defaultFileName, Generator *g)
 
 void HelpProjectWriter::readSelectors(SubProject &subproject, const QStringList &selectors)
 {
-    QHash<QString, Node::NodeType> typeHash;
-    typeHash["namespace"] = Node::Namespace;
-    typeHash["class"] = Node::Class;
-    typeHash["struct"] = Node::Struct;
-    typeHash["union"] = Node::Union;
-    typeHash["header"] = Node::HeaderFile;
-    typeHash["headerfile"] = Node::HeaderFile;
-    typeHash["doc"] = Node::Page; // Unused (supported but ignored as a prefix)
-    typeHash["fake"] = Node::Page; // Unused (supported but ignored as a prefix)
-    typeHash["page"] = Node::Page;
-    typeHash["enum"] = Node::Enum;
-    typeHash["example"] = Node::Example;
-    typeHash["externalpage"] = Node::ExternalPage;
-    typeHash["typedef"] = Node::Typedef;
-    typeHash["typealias"] = Node::TypeAlias;
-    typeHash["function"] = Node::Function;
-    typeHash["property"] = Node::Property;
-    typeHash["variable"] = Node::Variable;
-    typeHash["group"] = Node::Group;
-    typeHash["module"] = Node::Module;
-    typeHash["qmlmodule"] = Node::QmlModule;
-    typeHash["qmlproperty"] = Node::QmlProperty;
-    typeHash["qmlclass"] = Node::QmlType; // Legacy alias for 'qmltype'
-    typeHash["qmltype"] = Node::QmlType;
-    typeHash["qmlbasictype"] = Node::QmlValueType; // Legacy alias for 'qmlvaluetype'
-    typeHash["qmlvaluetype"] = Node::QmlValueType;
+    QHash<QString, NodeType> typeHash;
+    typeHash["namespace"] = NodeType::Namespace;
+    typeHash["class"] = NodeType::Class;
+    typeHash["struct"] = NodeType::Struct;
+    typeHash["union"] = NodeType::Union;
+    typeHash["header"] = NodeType::HeaderFile;
+    typeHash["headerfile"] = NodeType::HeaderFile;
+    typeHash["doc"] = NodeType::Page; // Unused (supported but ignored as a prefix)
+    typeHash["fake"] = NodeType::Page; // Unused (supported but ignored as a prefix)
+    typeHash["page"] = NodeType::Page;
+    typeHash["enum"] = NodeType::Enum;
+    typeHash["example"] = NodeType::Example;
+    typeHash["externalpage"] = NodeType::ExternalPage;
+    typeHash["typedef"] = NodeType::Typedef;
+    typeHash["typealias"] = NodeType::TypeAlias;
+    typeHash["function"] = NodeType::Function;
+    typeHash["property"] = NodeType::Property;
+    typeHash["variable"] = NodeType::Variable;
+    typeHash["group"] = NodeType::Group;
+    typeHash["module"] = NodeType::Module;
+    typeHash["none"] = NodeType::NoType;
+    typeHash["qmlmodule"] = NodeType::QmlModule;
+    typeHash["qmlproperty"] = NodeType::QmlProperty;
+    typeHash["qmlclass"] = NodeType::QmlType; // Legacy alias for 'qmltype'
+    typeHash["qmltype"] = NodeType::QmlType;
+    typeHash["qmlbasictype"] = NodeType::QmlValueType; // Legacy alias for 'qmlvaluetype'
+    typeHash["qmlvaluetype"] = NodeType::QmlValueType;
 
     for (const QString &selector : selectors) {
         QStringList pieces = selector.split(QLatin1Char(':'));
         // Remove doc: or fake: prefix
-        if (pieces.size() > 1 && typeHash.value(pieces[0].toLower()) == Node::Page)
+        if (pieces.size() > 1 && typeHash.value(pieces[0].toLower()) == NodeType::Page)
             pieces.takeFirst();
 
         QString typeName = pieces.takeFirst().toLower();
         if (!typeHash.contains(typeName))
             continue;
 
-        subproject.m_selectors << typeHash.value(typeName);
+        subproject.m_selectors << static_cast<unsigned char>(typeHash.value(typeName));
         if (!pieces.isEmpty()) {
             pieces = pieces[0].split(QLatin1Char(','));
             for (const auto &piece : std::as_const(pieces)) {
-                if (typeHash[typeName] == Node::Group
-                    || typeHash[typeName] == Node::Module
-                    || typeHash[typeName] == Node::QmlModule) {
+                if (typeHash[typeName] == NodeType::Group
+                    || typeHash[typeName] == NodeType::Module
+                    || typeHash[typeName] == NodeType::QmlModule) {
                     subproject.m_groups << piece.toLower();
                 }
             }
@@ -162,7 +164,7 @@ void HelpProjectWriter::addExtraFile(const QString &file)
 
 Keyword HelpProjectWriter::keywordDetails(const Node *node) const
 {
-    QString ref = m_gen->fullDocumentLocation(node, false);
+    QString ref = m_gen->fullDocumentLocation(node);
 
     if (node->parent() && !node->parent()->name().isEmpty()) {
         QString name = (node->isEnumType() || node->isTypedef())
@@ -202,7 +204,11 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
     if (!node->url().isEmpty() && !(project.m_includeIndexNodes && !node->url().startsWith("http")))
         return false;
 
-    if (node->isPrivate() || node->isInternal() || node->isDontDocument())
+    // Process (members of) unseen group nodes, i.e. nodes that use \ingroup <group_name> where
+    // \group group_name itself is not documented.
+    bool unseenGroup{node->isGroup() && !node->wasSeen()};
+
+    if ((node->isPrivate() || node->isInternal() || node->isDontDocument()) && !unseenGroup)
         return false;
 
     if (node->name().isEmpty())
@@ -216,15 +222,14 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
     // Only add nodes to the set for each subproject if they match a selector.
     // Those that match will be listed in the table of contents.
 
-    for (int i = 0; i < project.m_subprojects.size(); i++) {
-        SubProject subproject = project.m_subprojects[i];
+    for (auto &subproject : project.m_subprojects) {
         // No selectors: accept all nodes.
         if (subproject.m_selectors.isEmpty()) {
-            project.m_subprojects[i].m_nodes[objName] = node;
-        } else if (subproject.m_selectors.contains(node->nodeType())) {
+            subproject.m_nodes.insert(objName, node);
+        } else if (subproject.m_selectors.contains(static_cast<unsigned char>(node->nodeType()))) {
             // Add all group members for '[group|module|qmlmodule]:name' selector
             if (node->isCollectionNode()) {
-                if (project.m_subprojects[i].m_groups.contains(node->name().toLower())) {
+                if (subproject.m_groups.contains(node->name().toLower())) {
                     const auto *cn = static_cast<const CollectionNode *>(node);
                     const auto members = cn->members();
                     for (const Node *m : members) {
@@ -232,50 +237,58 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
                             continue;
                         QString memberName =
                                 m->isTextPageNode() ? m->fullTitle() : m->fullDocumentName();
-                        project.m_subprojects[i].m_nodes[memberName] = m;
+                        subproject.m_nodes.insert(memberName, m);
                     }
                     continue;
-                } else if (!project.m_subprojects[i].m_groups.isEmpty()) {
+                } else if (!subproject.m_groups.isEmpty()) {
                     continue; // Node does not represent specified group(s)
                 }
             } else if (node->isTextPageNode()) {
                 if (node->isExternalPage() || node->fullTitle().isEmpty())
                     continue;
             }
-            project.m_subprojects[i].m_nodes[objName] = node;
+            subproject.m_nodes.insert(objName, node);
         }
     }
 
+    auto appendDocKeywords = [&](const Node *n) {
+        for (const auto *kw : n->doc().keywords()) {
+                if (!kw->string().isEmpty()) {
+                    QStringList ref_parts = m_gen->fullDocumentLocation(n).split('#'_L1);
+                    // Use keyword's custom anchor if it has one
+                    if (kw->count() > 1) {
+                        if (ref_parts.count() > 1)
+                            ref_parts.pop_back();
+                        ref_parts << kw->string(1);
+                    }
+                    project.m_keywords.append(Keyword(kw->string(), kw->string(),
+                            ref_parts.join('#'_L1)));
+                }
+            }
+    };
+    // Unseen group nodes require no further processing as they have no documentation
+    if (unseenGroup)
+        return false;
+
     switch (node->nodeType()) {
 
-    case Node::Class:
-    case Node::Struct:
-    case Node::Union:
+    case NodeType::Class:
+    case NodeType::Struct:
+    case NodeType::Union:
         project.m_keywords.append(keywordDetails(node));
         break;
-    case Node::QmlType:
-    case Node::QmlValueType:
-        if (node->doc().hasKeywords()) {
-            const auto keywords = node->doc().keywords();
-            for (const Atom *keyword : keywords) {
-                if (!keyword->string().isEmpty()) {
-                    project.m_keywords.append(Keyword(keyword->string(), keyword->string(),
-                                                      m_gen->fullDocumentLocation(node, false)));
-                }
-                else
-                    node->doc().location().warning(
-                            QStringLiteral("Bad keyword in %1")
-                                    .arg(m_gen->fullDocumentLocation(node, false)));
-            }
-        }
+    case NodeType::QmlType:
+    case NodeType::QmlValueType:
+        appendDocKeywords(node);
         project.m_keywords.append(keywordDetails(node));
         break;
 
-    case Node::Namespace:
+    case NodeType::Namespace:
         project.m_keywords.append(keywordDetails(node));
         break;
 
-    case Node::Enum:
+    case NodeType::Enum:
+    case NodeType::QmlEnum:
         project.m_keywords.append(keywordDetails(node));
         {
             const auto *enumNode = static_cast<const EnumNode *>(node);
@@ -291,40 +304,27 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
                 } else {
                     name = id = item.name();
                 }
-                QString ref = m_gen->fullDocumentLocation(node, false);
-                project.m_keywords.append(Keyword(name, id, ref));
+                QString ref = m_gen->fullDocumentLocation(node);
+                project.m_keywords.append(Keyword(std::move(name), std::move(id), std::move(ref)));
             }
         }
         break;
 
-    case Node::Group:
-    case Node::Module:
-    case Node::QmlModule: {
-        const auto *cn = static_cast<const CollectionNode *>(node);
-        if (!cn->fullTitle().isEmpty()) {
-            if (cn->doc().hasKeywords()) {
-                const auto keywords = cn->doc().keywords();
-                for (const Atom *keyword : keywords) {
-                    if (!keyword->string().isEmpty()) {
-                        project.m_keywords.append(
-                                Keyword(keyword->string(), keyword->string(),
-                                        m_gen->fullDocumentLocation(node, false)));
-                    } else
-                        cn->doc().location().warning(
-                                QStringLiteral("Bad keyword in %1")
-                                        .arg(m_gen->fullDocumentLocation(node, false)));
-                }
-            }
+    case NodeType::Group:
+    case NodeType::Module:
+    case NodeType::QmlModule: {
+        if (!node->fullTitle().isEmpty()) {
+            appendDocKeywords(node);
             project.m_keywords.append(keywordDetails(node));
         }
     } break;
 
-    case Node::Property:
-    case Node::QmlProperty:
+    case NodeType::Property:
+    case NodeType::QmlProperty:
         project.m_keywords.append(keywordDetails(node));
         break;
 
-    case Node::Function: {
+    case NodeType::Function: {
         const auto *funcNode = static_cast<const FunctionNode *>(node);
 
         /*
@@ -351,41 +351,28 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
         if (node->parent())
             project.m_memberStatus[node->parent()].insert(node->status());
     } break;
-    case Node::TypeAlias:
-    case Node::Typedef: {
+    case NodeType::TypeAlias:
+    case NodeType::Typedef: {
         const auto *typedefNode = static_cast<const TypedefNode *>(node);
         Keyword typedefDetails = keywordDetails(node);
         const EnumNode *enumNode = typedefNode->associatedEnum();
         // Use the location of any associated enum node in preference
         // to that of the typedef.
         if (enumNode)
-            typedefDetails.m_ref = m_gen->fullDocumentLocation(enumNode, false);
+            typedefDetails.m_ref = m_gen->fullDocumentLocation(enumNode);
 
         project.m_keywords.append(typedefDetails);
     } break;
 
-    case Node::Variable: {
+    case NodeType::Variable: {
         project.m_keywords.append(keywordDetails(node));
     } break;
 
         // Page nodes (such as manual pages) contain subtypes, titles and other
         // attributes.
-    case Node::Page: {
-        const auto *pn = static_cast<const PageNode *>(node);
-        if (!pn->fullTitle().isEmpty()) {
-            if (pn->doc().hasKeywords()) {
-                const auto keywords = pn->doc().keywords();
-                for (const Atom *keyword : keywords) {
-                    if (!keyword->string().isEmpty()) {
-                        project.m_keywords.append(
-                                Keyword(keyword->string(), keyword->string(),
-                                        m_gen->fullDocumentLocation(node, false)));
-                    } else {
-                        QString loc = m_gen->fullDocumentLocation(node, false);
-                        pn->doc().location().warning(QStringLiteral("Bad keyword in %1").arg(loc));
-                    }
-                }
-            }
+    case NodeType::Page: {
+        if (!node->fullTitle().isEmpty()) {
+            appendDocKeywords(node);
             project.m_keywords.append(keywordDetails(node));
         }
         break;
@@ -430,6 +417,11 @@ void HelpProjectWriter::generateSections(HelpProject &project, QXmlStreamWriter
             // Skip related non-members adopted by some other aggregate
             if (child->parent() != aggregate)
                 continue;
+            // Process unseen group nodes (even though they're marked internal)
+            if (child->isGroup() && !child->wasSeen()) {
+                childSet << child;
+                continue;
+            }
             if (child->isIndexNode() || child->isPrivate())
                 continue;
             if (child->isTextPageNode()) {
@@ -460,19 +452,6 @@ void HelpProjectWriter::generate()
         generateProject(project);
 }
 
-void HelpProjectWriter::writeHashFile(QFile &file)
-{
-    QCryptographicHash hash(QCryptographicHash::Sha1);
-    hash.addData(&file);
-
-    QFile hashFile(file.fileName() + ".sha1");
-    if (!hashFile.open(QFile::WriteOnly))
-        return;
-
-    hashFile.write(hash.result().toHex());
-    hashFile.close();
-}
-
 void HelpProjectWriter::writeSection(QXmlStreamWriter &writer, const QString &path,
                                      const QString &value)
 {
@@ -487,7 +466,7 @@ void HelpProjectWriter::writeSection(QXmlStreamWriter &writer, const QString &pa
  */
 void HelpProjectWriter::addMembers(HelpProject &project, QXmlStreamWriter &writer, const Node *node)
 {
-    QString href = m_gen->fullDocumentLocation(node, false);
+    QString href = m_gen->fullDocumentLocation(node);
     href = href.left(href.size() - 5);
     if (href.isEmpty())
         return;
@@ -498,7 +477,7 @@ void HelpProjectWriter::addMembers(HelpProject &project, QXmlStreamWriter &write
 
     // Do not generate a 'List of all members' for namespaces or header files,
     // but always generate it for derived classes and QML types (but not QML value types)
-    if (!node->isNamespace() && !node->isHeader() && !node->isQmlBasicType()
+    if (!node->isNamespace() && !node->isHeader() && !node->isQmlBasicType() && !node->isWrapper()
         && (derivedClass || node->isQmlType() || !project.m_memberStatus[node].isEmpty())) {
         QString membersPath = href + QStringLiteral("-members.html");
         writeSection(writer, membersPath, QStringLiteral("List of all members"));
@@ -511,16 +490,16 @@ void HelpProjectWriter::addMembers(HelpProject &project, QXmlStreamWriter &write
 
 void HelpProjectWriter::writeNode(HelpProject &project, QXmlStreamWriter &writer, const Node *node)
 {
-    QString href = m_gen->fullDocumentLocation(node, false);
+    QString href = m_gen->fullDocumentLocation(node);
     QString objName = node->name();
 
     switch (node->nodeType()) {
 
-    case Node::Class:
-    case Node::Struct:
-    case Node::Union:
-    case Node::QmlType:
-    case Node::QmlValueType: {
+    case NodeType::Class:
+    case NodeType::Struct:
+    case NodeType::Union:
+    case NodeType::QmlType:
+    case NodeType::QmlValueType: {
         QString typeStr = m_gen->typeString(node);
         if (!typeStr.isEmpty())
             typeStr[0] = typeStr[0].toTitleCase();
@@ -537,20 +516,20 @@ void HelpProjectWriter::writeNode(HelpProject &project, QXmlStreamWriter &writer
         writer.writeEndElement(); // section
     } break;
 
-    case Node::Namespace:
-        writeSection(writer, href, objName);
+    case NodeType::Namespace:
+        writeSection(writer, href, "%1 Namespace Reference"_L1.arg(objName));
         break;
 
-    case Node::Example:
-    case Node::HeaderFile:
-    case Node::Page:
-    case Node::Group:
-    case Node::Module:
-    case Node::QmlModule: {
+    case NodeType::Example:
+    case NodeType::HeaderFile:
+    case NodeType::Page:
+    case NodeType::Group:
+    case NodeType::Module:
+    case NodeType::QmlModule: {
         writer.writeStartElement("section");
         writer.writeAttribute("ref", href);
         writer.writeAttribute("title", node->fullTitle());
-        if (node->nodeType() == Node::HeaderFile)
+        if (node->nodeType() == NodeType::HeaderFile)
             addMembers(project, writer, node);
         writer.writeEndElement(); // section
     } break;
@@ -625,7 +604,7 @@ void HelpProjectWriter::generateProject(HelpProject &project)
         node = m_qdb->findNodeByNameAndType(QStringList("index.html"), &Node::isPageNode);
     QString indexPath;
     if (node)
-        indexPath = m_gen->fullDocumentLocation(node, false);
+        indexPath = m_gen->fullDocumentLocation(node);
     else
         indexPath = "index.html";
     writer.writeAttribute("ref", indexPath);
@@ -667,7 +646,7 @@ void HelpProjectWriter::generateProject(HelpProject &project)
 
                             const Node *page = m_qdb->findNodeForTarget(atom->string(), nullptr);
                             writer.writeStartElement("section");
-                            QString indexPath = m_gen->fullDocumentLocation(page, false);
+                            QString indexPath = m_gen->fullDocumentLocation(page);
                             writer.writeAttribute("ref", indexPath);
                             writer.writeAttribute("title", atom->linkText());
 
@@ -682,14 +661,17 @@ void HelpProjectWriter::generateProject(HelpProject &project)
                     atom = atom->next();
                 }
             } else
-                rootNode->doc().location().warning(
-                        QStringLiteral("Failed to find index: %1").arg(subproject.m_indexTitle));
+                Config::instance().location().warning(
+                        "Failed to find %1.indexTitle '%2'"_L1.arg(subproject.m_prefix, subproject.m_indexTitle));
 
         } else {
 
             writer.writeStartElement("section");
             QString indexPath = m_gen->fullDocumentLocation(
-                    m_qdb->findNodeForTarget(subproject.m_indexTitle, nullptr), false);
+                    m_qdb->findNodeForTarget(subproject.m_indexTitle, nullptr));
+            if (indexPath.isEmpty() && !subproject.m_indexTitle.isEmpty())
+                Config::instance().location().warning(
+                        "Failed to find %1.indexTitle '%2'"_L1.arg(subproject.m_prefix, subproject.m_indexTitle));
             writer.writeAttribute("ref", indexPath);
             writer.writeAttribute("title", subproject.m_title);
 
@@ -779,10 +761,6 @@ void HelpProjectWriter::generateProject(HelpProject &project)
     writer.writeEndElement(); // QtHelpProject
     writer.writeEndDocument();
     file.close();
-    if (file.open(QFile::ReadOnly)) {
-        writeHashFile(file);
-        file.close();
-    }
 }
 
 QT_END_NAMESPACE
