@@ -34,6 +34,28 @@ QT_BEGIN_NAMESPACE
   is being generated.
  */
 
+/*!
+  \class TargetRec
+  \brief A record of a linkable target within the documentation.
+*/
+
+/*!
+    \enum TargetRec::TargetType
+
+    A type of a linkable target record.
+
+    \value Unknown
+           Unknown/target not set.
+    \value Target
+           A location marked with a \\target command.
+    \value Keyword
+           A location marked with a \\keyword command.
+    \value Contents
+           A table of contents item (section title).
+    \value ContentsKeyword
+           A \\keyword tied to a section title.
+*/
+
 /*!
   Constructs a Tree. \a qdb is the pointer to the singleton
   qdoc database that is constructing the tree. This might not
@@ -166,7 +188,7 @@ void Tree::resolveBaseClasses(Aggregate *n)
     for (auto it = n->constBegin(); it != n->constEnd(); ++it) {
         if ((*it)->isClassNode()) {
             auto *cn = static_cast<ClassNode *>(*it);
-            QList<RelatedClass> &bases = cn->baseClasses();
+            QList<RelatedClass> &bases = cn->baseClasses_mutable();
             for (auto &base : bases) {
                 if (base.m_node == nullptr) {
                     Node *n = m_qdb->findClassNode(base.m_path);
@@ -283,7 +305,7 @@ void Tree::resolveCppToQmlLinks()
             auto *qcn = static_cast<QmlTypeNode *>(child);
             auto *cn = const_cast<ClassNode *>(qcn->classNode());
             if (cn)
-                cn->setQmlElement(qcn);
+                cn->insertQmlNativeType(qcn);
         }
     }
 }
@@ -327,20 +349,14 @@ void Tree::resolveSince(Aggregate &aggregate)
 */
 void Tree::resolveEnumValueSince(EnumNode &en)
 {
-    auto findNextAtom = [](const Atom *a, Atom::AtomType t) {
-        while (a && a->type() != t)
-            a = a->next();
-        return a;
-    };
-
     const QStringList enumItems{en.doc().enumItemNames()};
     const Atom *atom = en.doc().body().firstAtom();
     if (!atom)
         return;
-    while ((atom = findNextAtom(atom, Atom::ListTagLeft))) {
+    while ((atom = atom->find(Atom::ListTagLeft))) {
         if (atom = atom->next(); !atom)
             break;
-        if (auto val = atom->string(); enumItems.contains(val)) {
+        if (const auto &val = atom->string(); enumItems.contains(val)) {
             if (atom = atom->next(); atom && atom->next(Atom::SinceTagLeft))
                 en.setSince(val, atom->next()->next()->string());
         }
@@ -458,7 +474,7 @@ Node *Tree::findNodeRecursive(const QStringList &path, int pathIndex, const Node
   (Unknown).
  */
 const Node *Tree::findNodeForTarget(const QStringList &path, const QString &target,
-                                    const Node *start, int flags, Node::Genus genus,
+                                    const Node *start, int flags, Genus genus,
                                     QString &ref, TargetRec::TargetType *targetType) const
 {
     const Node *node = nullptr;
@@ -473,7 +489,7 @@ const Node *Tree::findNodeForTarget(const QStringList &path, const QString &targ
         return n;
     };
 
-    if (genus == Node::DontCare || genus == Node::DOC) {
+    if (genus == Genus::DontCare || genus == Genus::DOC) {
         if (node = findPageNodeByTitle(path.at(0)); node) {
             if (node = set_ref_from_target(node); node)
                 return node;
@@ -503,7 +519,7 @@ const Node *Tree::findNodeForTarget(const QStringList &path, const QString &targ
       the type.
     */
     int path_idx = 0;
-    if ((genus == Node::QML || genus == Node::DontCare)
+    if ((genus == Genus::QML || genus == Genus::DontCare)
         && path.size() >= 2 && !path[0].isEmpty()) {
         if (auto *qcn = lookupQmlType(path.sliced(0, 2).join(QLatin1String("::"))); qcn) {
             current = qcn;
@@ -555,7 +571,7 @@ const Node *Tree::findNodeForTarget(const QStringList &path, const QString &targ
   most often the root of this Tree.
  */
 const Node *Tree::matchPathAndTarget(const QStringList &path, int idx, const QString &target,
-                                     const Node *node, int flags, Node::Genus genus,
+                                     const Node *node, int flags, Genus genus,
                                      QString &ref) const
 {
     /*
@@ -579,7 +595,7 @@ const Node *Tree::matchPathAndTarget(const QStringList &path, int idx, const QSt
         NodeVector nodes;
         static_cast<const Aggregate *>(node)->findChildren(name, nodes);
         for (const auto *child : std::as_const(nodes)) {
-            if (genus != Node::DontCare && !(genus & child->genus()))
+            if (genus != Genus::DontCare && !(hasCommonGenusType(genus, child->genus())))
                 continue;
             const Node *t = matchPathAndTarget(path, idx + 1, target, child, flags, genus, ref);
             if (t && !t->isPrivate())
@@ -593,7 +609,7 @@ const Node *Tree::matchPathAndTarget(const QStringList &path, int idx, const QSt
         if (enumNode)
             return enumNode;
     }
-    if (((genus == Node::CPP) || (genus == Node::DontCare)) && node->isClassNode()
+    if (((genus == Genus::CPP) || (genus == Genus::DontCare)) && node->isClassNode()
         && (flags & SearchBaseClasses)) {
         const ClassList bases = allBaseClasses(static_cast<const ClassNode *>(node));
         for (const auto *base : bases) {
@@ -616,7 +632,7 @@ const Node *Tree::matchPathAndTarget(const QStringList &path, int idx, const QSt
   restrict the search.
  */
 const Node *Tree::findNode(const QStringList &path, const Node *start, int flags,
-                           Node::Genus genus) const
+                           Genus genus) const
 {
     const Node *current = start;
     if (current == nullptr)
@@ -635,7 +651,7 @@ const Node *Tree::findNode(const QStringList &path, const Node *start, int flags
           If the answer is yes, the reference identifies a QML
           type node.
         */
-        if (((genus == Node::QML) || (genus == Node::DontCare)) && (path.size() >= 2)
+        if (((genus == Genus::QML) || (genus == Genus::DontCare)) && (path.size() >= 2)
             && !path[0].isEmpty()) {
             QmlTypeNode *qcn = lookupQmlType(QString(path[0] + "::" + path[1]));
             if (qcn != nullptr) {
@@ -664,7 +680,7 @@ const Node *Tree::findNode(const QStringList &path, const Node *start, int flags
                 return enumNode;
 
 
-            if (!next && ((genus == Node::CPP) || (genus == Node::DontCare))
+            if (!next && ((genus == Genus::CPP) || (genus == Genus::DontCare))
                 && node->isClassNode() && (flags & SearchBaseClasses)) {
                 const ClassList bases = allBaseClasses(static_cast<const ClassNode *>(node));
                 for (const auto *base : bases) {
@@ -753,88 +769,167 @@ void Tree::insertTarget(const QString &name, const QString &title, TargetRec::Ta
 }
 
 /*!
+    \internal
+
+    \a root is the root node of the tree to resolve targets for. This function
+    traverses the tree starting from the root node and processes each child
+    node. If the child node is an aggregate node, this function is called
+    recursively on the child node.
  */
 void Tree::resolveTargets(Aggregate *root)
 {
     for (auto *child : root->childNodes()) {
-        if (child->isTextPageNode()) {
-            auto *node = static_cast<PageNode *>(child);
-            QString key = node->title();
-            if (!key.isEmpty()) {
-                if (key.contains(QChar(' ')))
-                    key = Utilities::asAsciiPrintable(key);
-                QList<PageNode *> nodes = m_pageNodesByTitle.values(key);
-                bool alreadyThere = false;
-                if (!nodes.empty()) {
-                    for (const auto &node_ : nodes) {
-                        if (node_->isExternalPage()) {
-                            if (node->name() == node_->name()) {
-                                alreadyThere = true;
-                                break;
-                            }
-                        }
-                    }
-                }
-                if (!alreadyThere)
-                    m_pageNodesByTitle.insert(key, node);
-            }
-        }
+        addToPageNodeByTitleMap(child);
+        populateTocSectionTargetMap(child);
+        addKeywordsToTargetMaps(child);
+        addTargetsToTargetMap(child);
 
-        if (child->doc().hasTableOfContents()) {
-            const QList<Atom *> &toc = child->doc().tableOfContents();
-            for (Atom *i : toc) {
-                QString ref = refForAtom(i);
-                QString title = Text::sectionHeading(i).toString();
-                if (!ref.isEmpty() && !title.isEmpty()) {
-                    QString key = Utilities::asAsciiPrintable(title);
-                    auto *target = new TargetRec(ref, TargetRec::Contents, child, 3);
-                    m_nodesByTargetRef.insert(key, target);
-                    m_nodesByTargetTitle.insert(title, target);
-                }
-            }
-        }
-        if (child->doc().hasKeywords()) {
-            const QList<Atom *> &keywords = child->doc().keywords();
-            for (Atom *i : keywords) {
-                QString ref = refForAtom(i);
-                QString title = i->string();
-                if (!ref.isEmpty() && !title.isEmpty()) {
-                    auto *target = new TargetRec(ref, TargetRec::Keyword, child, 1);
-                    m_nodesByTargetRef.insert(Utilities::asAsciiPrintable(title), target);
-                    m_nodesByTargetTitle.insert(title, target);
-                }
-            }
-        }
-        if (child->doc().hasTargets()) {
-            const QList<Atom *> &targets = child->doc().targets();
-            for (Atom *i : targets) {
-                QString ref = refForAtom(i);
-                QString title = i->string();
-                if (!ref.isEmpty() && !title.isEmpty()) {
-                    QString key = Utilities::asAsciiPrintable(title);
-                    auto *target = new TargetRec(ref, TargetRec::Target, child, 2);
-                    m_nodesByTargetRef.insert(key, target);
-                    m_nodesByTargetTitle.insert(title, target);
-                }
-            }
-        }
         if (child->isAggregate())
             resolveTargets(static_cast<Aggregate *>(child));
     }
 }
 
+/*!
+    \internal
+
+    Updates the target maps for targets associated with the given \a node.
+ */
+void Tree::addTargetsToTargetMap(Node *node) {
+    if (!node || !node->doc().hasTargets())
+        return;
+
+    for (Atom *i : std::as_const(node->doc().targets())) {
+        const QString ref = refForAtom(i);
+        const QString title = i->string();
+        if (!ref.isEmpty() && !title.isEmpty()) {
+            QString key = Utilities::asAsciiPrintable(title);
+            auto *target = new TargetRec(std::move(ref), TargetRec::Target, node, 2);
+            m_nodesByTargetRef.insert(key, target);
+            m_nodesByTargetTitle.insert(title, target);
+        }
+    }
+}
+
+/*
+    If atom \a a is immediately followed by a
+    section title (\section1..\section4 command),
+    returns the SectionLeft atom; otherwise nullptr.
+*/
+static const Atom *nextSection(const Atom *a)
+{
+    while (a && a->next(Atom::SectionRight))
+        a = a->next(); // skip closing section atoms
+    return a ? a->next(Atom::SectionLeft) : nullptr;
+}
+
+/*!
+    \internal
+
+    Updates the target maps for keywords associated with the given \a node.
+ */
+void Tree::addKeywordsToTargetMaps(Node *node) {
+    if (!node->doc().hasKeywords())
+        return;
+
+    for (Atom *i : std::as_const(node->doc().keywords())) {
+        QString ref = refForAtom(i);
+        QString title = i->string();
+        if (!ref.isEmpty() && !title.isEmpty()) {
+            auto *target = new TargetRec(ref, nextSection(i) ? TargetRec::ContentsKeyword : TargetRec::Keyword, node, 1);
+            m_nodesByTargetRef.insert(Utilities::asAsciiPrintable(title), target);
+            m_nodesByTargetTitle.insert(title, target);
+            if (!target->isEmpty())
+                i->append(target->m_ref);
+        }
+    }
+}
+
+/*!
+    \internal
+
+    Populates the map of targets for each section in the table of contents for
+    the given \a node while ensuring that each target has a unique reference.
+ */
+void Tree::populateTocSectionTargetMap(Node *node) {
+    if (!node || !node->doc().hasTableOfContents())
+        return;
+
+    QStack<Atom *> tocLevels;
+    QSet<QString> anchors;
+
+    qsizetype index = 0;
+
+    for (Atom *atom: std::as_const(node->doc().tableOfContents())) {
+        while (!tocLevels.isEmpty() && tocLevels.top()->string().toInt() >= atom->string().toInt())
+            tocLevels.pop();
+
+        tocLevels.push(atom);
+
+        QString ref = refForAtom(atom);
+        const QString &title = Text::sectionHeading(atom).toString();
+        if (ref.isEmpty() || title.isEmpty())
+            continue;
+
+        if (anchors.contains(ref)) {
+            QStringList refParts;
+            for (const auto tocLevel : tocLevels)
+                 refParts << refForAtom(tocLevel);
+
+            refParts << QString::number(index);
+            ref = refParts.join(QLatin1Char('-'));
+        }
+
+        anchors.insert(ref);
+        if (atom->next(Atom::SectionHeadingLeft))
+            atom->next()->append(ref);
+        ++index;
+
+        const QString &key = Utilities::asAsciiPrintable(title);
+        auto *target = new TargetRec(ref, TargetRec::Contents, node, 3);
+        m_nodesByTargetRef.insert(key, target);
+        m_nodesByTargetTitle.insert(title, target);
+    }
+}
+
+/*!
+    \internal
+
+    Checks if the \a node's title is registered in the page nodes by title map.
+    If not, it stores the page node in the map.
+ */
+void Tree::addToPageNodeByTitleMap(Node *node) {
+    if (!node || !node->isTextPageNode())
+        return;
+
+    auto *pageNode = static_cast<PageNode *>(node);
+    QString key = pageNode->title();
+    if (key.isEmpty())
+        return;
+
+    if (key.contains(QChar(' ')))
+        key = Utilities::asAsciiPrintable(key);
+    const QList<PageNode *> nodes = m_pageNodesByTitle.values(key);
+
+    bool alreadyThere = std::any_of(nodes.cbegin(), nodes.cend(), [&](const auto &knownNode) {
+        return knownNode->isExternalPage() && knownNode->name() == pageNode->name();
+    });
+
+    if (!alreadyThere)
+        m_pageNodesByTitle.insert(key, pageNode);
+}
+
 /*!
   Searches for a \a target anchor, matching the given \a genus, and returns
   the associated TargetRec instance.
  */
-const TargetRec *Tree::findUnambiguousTarget(const QString &target, Node::Genus genus) const
+const TargetRec *Tree::findUnambiguousTarget(const QString &target, Genus genus) const
 {
     auto findBestCandidate = [&](const TargetMap &tgtMap, const QString &key) {
         TargetRec *best = nullptr;
         auto [it, end] = tgtMap.equal_range(key);
         while (it != end) {
             TargetRec *candidate = it.value();
-            if ((genus == Node::DontCare) || (genus & candidate->genus())) {
+            if ((genus == Genus::DontCare) || (hasCommonGenusType(genus, candidate->genus()))) {
                 if (!best || (candidate->m_priority < best->m_priority))
                     best = candidate;
             }
@@ -888,17 +983,32 @@ const PageNode *Tree::findPageNodeByTitle(const QString &title) const
 
 /*!
   Returns a canonical title for the \a atom, if the \a atom
-  is a SectionLeft or a Target.
+  is a SectionLeft, SectionHeadingLeft, Keyword, or Target.
+
+  If a target or a keyword is immediately followed by a
+  section, the former adopts the title (ref) of the latter.
  */
 QString Tree::refForAtom(const Atom *atom)
 {
-    if (atom) {
-        if (atom->type() == Atom::SectionLeft)
-            return Utilities::asAsciiPrintable(Text::sectionHeading(atom).toString());
-        if ((atom->type() == Atom::Target) || (atom->type() == Atom::Keyword))
-            return Utilities::asAsciiPrintable(atom->string());
+    Q_ASSERT(atom);
+
+    switch (atom->type()) {
+    case Atom::SectionLeft:
+        atom = atom->next();
+        [[fallthrough]];
+    case Atom::SectionHeadingLeft:
+        if (atom->count() == 2)
+            return atom->string(1);
+        return Utilities::asAsciiPrintable(Text::sectionHeading(atom).toString());
+    case Atom::Target:
+        [[fallthrough]];
+    case Atom::Keyword:
+        if (const auto *section = nextSection(atom))
+            return refForAtom(section);
+        return Utilities::asAsciiPrintable(atom->string());
+    default:
+        return {};
     }
-    return QString();
 }
 
 /*!
@@ -923,14 +1033,14 @@ QString Tree::refForAtom(const Atom *atom)
   Returns a pointer to the collection map specified by \a type.
   Returns null if \a type is not specified.
  */
-CNMap *Tree::getCollectionMap(Node::NodeType type)
+CNMap *Tree::getCollectionMap(NodeType type)
 {
     switch (type) {
-    case Node::Group:
+    case NodeType::Group:
         return &m_groups;
-    case Node::Module:
+    case NodeType::Module:
         return &m_modules;
-    case Node::QmlModule:
+    case NodeType::QmlModule:
         return &m_qmlModules;
     default:
         break;
@@ -944,7 +1054,7 @@ CNMap *Tree::getCollectionMap(Node::NodeType type)
   to it is returned. If a collection is not found, null is
   returned.
  */
-CollectionNode *Tree::getCollection(const QString &name, Node::NodeType type)
+CollectionNode *Tree::getCollection(const QString &name, NodeType type)
 {
     CNMap *map = getCollectionMap(type);
     if (map) {
@@ -969,7 +1079,7 @@ CollectionNode *Tree::getCollection(const QString &name, Node::NodeType type)
   If it is \c{DontCare}, 0 is returned, which is a programming
   error.
  */
-CollectionNode *Tree::findCollection(const QString &name, Node::NodeType type)
+CollectionNode *Tree::findCollection(const QString &name, NodeType type)
 {
     CNMap *m = getCollectionMap(type);
     if (!m) // error
@@ -1122,10 +1232,10 @@ void Tree::insertQmlType(const QString &key, QmlTypeNode *n)
   \a relative is ull, the search begins at the tree root.
  */
 const FunctionNode *Tree::findFunctionNode(const QStringList &path, const Parameters &parameters,
-                                           const Node *relative, Node::Genus genus) const
+                                           const Node *relative, Genus genus) const
 {
     if (path.size() == 3 && !path[0].isEmpty()
-        && ((genus == Node::QML) || (genus == Node::DontCare))) {
+        && ((genus == Genus::QML) || (genus == Genus::DontCare))) {
         QmlTypeNode *qcn = lookupQmlType(QString(path[0] + "::" + path[1]));
         if (qcn == nullptr) {
             QStringList p(path[1]);
@@ -1139,8 +1249,8 @@ const FunctionNode *Tree::findFunctionNode(const QStringList &path, const Parame
 
     if (relative == nullptr)
         relative = root();
-    else if (genus != Node::DontCare) {
-        if (!(genus & relative->genus()))
+    else if (genus != Genus::DontCare) {
+        if (!(hasCommonGenusType(genus, relative->genus())))
             relative = root();
     }
 
